#pragma once

#include <stdint.h>

// VT = Vector type
#define IDT_VT_UNDEFINED 0
#define IDT_VT_EXCEPTION 1
#define IDT_VT_IRQ 2

typedef struct {
  uint64_t r15;
  uint64_t r14;
  uint64_t r13;
  uint64_t r12;
  uint64_t r11;
  uint64_t r10;
  uint64_t r9;
  uint64_t r8;
  uint64_t rdi;
  uint64_t rsi;
  uint64_t rbp;
  uint64_t rbx;
  uint64_t rdx;
  uint64_t rcx;
  uint64_t rax;
  uint64_t int_no;
  uint64_t err_code;
  uint64_t rip;
  uint64_t cs;
  uint64_t rflags;
  uint64_t rsp;
  uint64_t ss;
} __attribute__((packed)) registers_t;

typedef struct stackframe {
  struct stackframe *rbp;
  uint64_t rip;
} stackframe_t;

typedef struct {
  uint16_t isr_low;   // The lower 16 bits of the ISR's address
  uint16_t kernel_cs; // The GDT segment selector that the CPU will load into CS
                      // before calling the ISR
  uint8_t ist; // The IST in the TSS that the CPU will load into RSP; set to
               // zero for now
  uint8_t attributes; // Type and attributes; see the IDT page
  uint16_t
      isr_mid; // The higher 16 bits of the lower 32 bits of the ISR's address
  uint32_t isr_high; // The higher 32 bits of the ISR's address
  uint32_t reserved; // Set to zero
} __attribute__((packed)) idt_entry_t;

typedef struct {
  uint16_t limit;
  uint64_t base;
} __attribute__((packed)) idtr_t;

typedef void (*idt_irq_handler_t)(registers_t *regs);

void idt_register_irq(uint8_t vector, void *isr);
void idt_set_descriptor(uint8_t vector, void *isr, uint8_t flags);
void idt_init(void);